@@ -9,3 +9,7 @@ from __future__ import annotations
 from ._version import version as __version__
 
 __all__ = ["__version__"]
+
+from .index import IDCClient
+
+_ = IDCClient
