@@ -5,18 +5,34 @@ import os
 import tempfile
 import unittest
 from itertools import product
+from pathlib import Path
 
 import pandas as pd
 import pytest
+import requests
 from click.testing import CliRunner
-from idc_index import cli, index
+from idc_index import IDCClient, cli
 
 # Run tests using the following command from the root of the repository:
 # python -m unittest -vv tests/idcindex.py
+#
+# run specific tests with this:
+# pytest ./tests/idcindex.py::TestIDCClient.test_download_dicom_instance
 
 logging.basicConfig(level=logging.DEBUG)
 
 
+def remote_file_exists(url):
+    try:
+        response = requests.head(url, allow_redirects=True)
+        # Check if the status code indicates success
+        return response.status_code == 200
+    except requests.RequestException as e:
+        # Handle any exceptions (e.g., network issues)
+        print(f"An error occurred: {e}")
+        return False
+
+
 @pytest.fixture(autouse=True)
 def _change_test_dir(request, monkeypatch):
     monkeypatch.chdir(request.fspath.dirname)
@@ -24,9 +40,10 @@ def _change_test_dir(request, monkeypatch):
 
 class TestIDCClient(unittest.TestCase):
     def setUp(self):
-        self.client = index.IDCClient()
+        self.client = IDCClient()
         self.download_from_manifest = cli.download_from_manifest
         self.download_from_selection = cli.download_from_selection
+        self.download = cli.download
 
         logger = logging.getLogger("idc_index")
         logger.setLevel(logging.DEBUG)
@@ -172,6 +189,17 @@ class TestIDCClient(unittest.TestCase):
             )
             self.assertEqual(sum([len(files) for r, d, files in os.walk(temp_dir)]), 3)
 
+    def test_download_dicom_instance(self):
+        i = IDCClient()
+        i.fetch_index("sm_instance_index")
+        with tempfile.TemporaryDirectory() as temp_dir:
+            self.client.download_dicom_instance(
+                sopInstanceUID="1.3.6.1.4.1.5962.99.1.528744472.1087975700.1641206284312.14.0",
+                downloadDir=temp_dir,
+            )
+
+            self.assertEqual(sum([len(files) for r, d, files in os.walk(temp_dir)]), 1)
+
     def test_download_with_template(self):
         dirTemplateValues = [
             None,
@@ -421,6 +449,123 @@ class TestIDCClient(unittest.TestCase):
             )
             assert len(os.listdir(temp_dir)) != 0
 
+    def test_singleton_attribute(self):
+        # singleton, initialized on first use
+        i1 = IDCClient.client()
+        i2 = IDCClient.client()
+
+        # new instances created via constructor (through init)
+        i3 = IDCClient()
+        i4 = self.client
+
+        # all must be not none
+        assert i1 is not None
+        assert i2 is not None
+        assert i3 is not None
+        assert i4 is not None
+
+        # singletons must return the same instance
+        assert i1 == i2
+
+        # new instances must be different
+        assert i1 != i3
+        assert i1 != i4
+        assert i3 != i4
+
+        # all must be instances of IDCClient
+        assert isinstance(i1, IDCClient)
+        assert isinstance(i2, IDCClient)
+        assert isinstance(i3, IDCClient)
+        assert isinstance(i4, IDCClient)
+
+    def test_cli_download(self):
+        runner = CliRunner()
+        with runner.isolated_filesystem():
+            result = runner.invoke(
+                self.download,
+                # StudyInstanceUID:
+                ["1.3.6.1.4.1.14519.5.2.1.7695.1700.114861588187429958687900856462"],
+            )
+            assert len(os.listdir(Path.cwd())) != 0
+
+        with runner.isolated_filesystem():
+            result = runner.invoke(
+                self.download,
+                # crdc_series_uuid:
+                ["e5c5c71d-62c4-4c50-a8a9-b6799c7f8dea"],
+            )
+            assert len(os.listdir(Path.cwd())) != 0
+
+    def test_prior_version_manifest(self):
+        # Define the values for each optional parameter
+        quiet_values = [True, False]
+        validate_manifest_values = [True, False]
+        show_progress_bar_values = [True, False]
+        use_s5cmd_sync_values = [True, False]
+        dirTemplateValues = [
+            None,
+            "%collection_id/%PatientID/%Modality/%StudyInstanceUID/%SeriesInstanceUID",
+            "%collection_id_%PatientID_%Modality_%StudyInstanceUID_%SeriesInstanceUID",
+        ]
+        # Generate all combinations of optional parameters
+        combinations = product(
+            quiet_values,
+            validate_manifest_values,
+            show_progress_bar_values,
+            use_s5cmd_sync_values,
+            dirTemplateValues,
+        )
+
+        # Test each combination
+        for (
+            quiet,
+            validate_manifest,
+            show_progress_bar,
+            use_s5cmd_sync,
+            dirTemplate,
+        ) in combinations:
+            with tempfile.TemporaryDirectory() as temp_dir:
+                self.client.download_from_manifest(
+                    manifestFile="./prior_version_manifest.s5cmd",
+                    downloadDir=temp_dir,
+                    quiet=quiet,
+                    validate_manifest=validate_manifest,
+                    show_progress_bar=show_progress_bar,
+                    use_s5cmd_sync=use_s5cmd_sync,
+                    dirTemplate=dirTemplate,
+                )
+
+                self.assertEqual(
+                    sum([len(files) for r, d, files in os.walk(temp_dir)]), 5
+                )
+
+    def test_list_indices(self):
+        i = IDCClient()
+        assert i.indices_overview  # assert that dict was created
+
+    def test_fetch_index(self):
+        i = IDCClient()
+        assert i.indices_overview["sm_index"]["installed"] is False
+        i.fetch_index("sm_index")
+        assert i.indices_overview["sm_index"]["installed"] is True
+        assert hasattr(i, "sm_index")
+
+    def test_indices_urls(self):
+        i = IDCClient()
+        for index in i.indices_overview:
+            if i.indices_overview[index]["url"] is not None:
+                assert remote_file_exists(i.indices_overview[index]["url"])
+
+    def test_clinical_index_install(self):
+        i = IDCClient()
+        assert i.indices_overview["clinical_index"]["installed"] is False
+        i.fetch_index("clinical_index")
+        assert i.indices_overview["clinical_index"]["installed"] is True
+        assert len(os.listdir(i.clinical_data_dir)) > 0
+
+        nlst_clinical = i.get_clinical_table("nlst_clinical")
+        assert nlst_clinical is not None
+
 
 if __name__ == "__main__":
     unittest.main()
